#include "Wire.h"
#include <MPU6050_light.h>

// Define the MPU6050 object
MPU6050 mpu(Wire);
#include<SoftwareSerial.h>
SoftwareSerial Btserial(15,14); // (rx,tx)

// Define variables for storing calibration offsets
float accelXOffset = 0;
float accelYOffset = 0;
float accelZOffset = 0;
float gyroXOffset = 0;
float gyroYOffset = 0;
float gyroZOffset = 0;

int gx = mpu.getAccX();
int gy = mpu.getAccY();
int gz = mpu.getAccZ();

int y = mpu.getGyroZ(); // yaw ---> z axis
int p = mpu.getGyroY(); // pitch ---> y axis
int r = mpu.getGyroX(); // roll ----> x axis

// Number of samples to collect for calibration
const int calibrationSamples = 10;

// Calibration of Five Flex Sensors with Display

int FLEX_PIN1 = A13; // Pinky
int FLEX_PIN2 = A5; // Ring
int FLEX_PIN3 = A3; // Middle
int FLEX_PIN4 = A1; // Index
int FLEX_PIN5 = A0; // Thumb

float voltage;
float resistance;

int sensorMin1 = 1023;
int sensorMax1 = 0;

int sensorMin2 = 1023;
int sensorMax2 = 0;

int sensorMin3 = 1023;
int sensorMax3 = 0;

int sensorMin4 = 1023;
int sensorMax4 = 0;

int sensorMin5 = 1023;
int sensorMax5 = 0;

int a; // pinky
int b; // ring
int c; // middle
int d; // index
int e; // thumb

void setup() 
{ // all function calls
  Serial.begin(9600);
  Wire.begin();
  Btserial.begin(9600);
  // Initialize I2C communication
  

  // Begin communication with MPU6050
  byte status = mpu.begin();
  if (status != 0)                      //mpu works on status==0
  {                                     //start of if
    Serial.print("MPU6050 initialization failed: ");
    Serial.println(status);
    while (1) 
     {
        } // Halt the program if initialization fails
  }        //end of if   
     
  
  Serial.println("Calibration Mode: Bend and Straighten Your Fingers");

  delay(3000); // Allow some time for the user to prepare

  calibrateFlexSensor(FLEX_PIN1, sensorMin1, sensorMax1, "Pinky");
  calibrateFlexSensor(FLEX_PIN2, sensorMin2, sensorMax2, "Ring");
  calibrateFlexSensor(FLEX_PIN3, sensorMin3, sensorMax3, "Middle");
  calibrateFlexSensor(FLEX_PIN4, sensorMin4, sensorMax4, "Index");
  calibrateFlexSensor(FLEX_PIN5, sensorMin5, sensorMax5, "Thumb");
  
  Serial.println("Calibration Completed");
  delay(1000);
  
  //Displaying the MIN/MAX values
  Serial.println("Sensor\tMin\tMax");
  displayCalibrationValues("Pinky", sensorMin1, sensorMax1);
  displayCalibrationValues("Ring", sensorMin2, sensorMax2);
  displayCalibrationValues("Middle", sensorMin3, sensorMax3);
  displayCalibrationValues("Index", sensorMin4, sensorMax4);
  displayCalibrationValues("Thumb", sensorMin5, sensorMax5);

  delay(2000);
    //end of if

  // Set accelerometer range to +-2g
//  mpu.setFullScaleAccelRange(MPU6050_RANGE_2_G);

  // Print a message to the user
  Serial.println("Starting calibration for MPU keep still...");

  // Collect calibration samples
  for (int i = 0; i < calibrationSamples; i++) 
  {                                                   //start of for (calibration for loop)
    mpu.update();

    // Accumulate offsets
    accelXOffset += mpu.getAccX();
    accelYOffset += mpu.getAccY();
    accelZOffset += mpu.getAccZ();
    gyroXOffset += mpu.getGyroX();
    gyroYOffset += mpu.getGyroY();
    gyroZOffset += mpu.getGyroZ();

    // Delay between samples
    delay(500);
  }                                                  //end of for (calibration for loop)

  // Calculate and apply offsets
  accelXOffset /= calibrationSamples;              //calc mean offset
  accelYOffset /= calibrationSamples;
  accelZOffset /= calibrationSamples;
  gyroXOffset /= calibrationSamples;
  gyroYOffset /= calibrationSamples;
  gyroZOffset /= calibrationSamples;

  // Set all offsets using a single call
  mpu.setAccOffsets(accelXOffset, accelYOffset, accelZOffset);
  mpu.setGyroOffsets(gyroXOffset, gyroYOffset, gyroZOffset);

  // Print calibration results
  Serial.println("Calibration completed!");
  Serial.print("Accel X offset: ");
  Serial.println(accelXOffset);
  Serial.print("\tAccel Y offset: ");
  Serial.println(accelYOffset);
  Serial.print("\tAccel Z offset: ");
  Serial.println(accelZOffset); 
 /* Serial.print("\tGyro X offset: ");
  Serial.println(gyroXOffset);
  Serial.print("\tGyro Y offset: ");
  Serial.println(gyroYOffset);
  Serial.print("\tGyro Z offset: ");
  Serial.println(gyroZOffset);  */


} //END OF SETUP

void loop() 
{ //START OF LOOP
  // Calibration is done in setup, no continuous action needed in loop because calibration is only one-time hence in setup not loop(repetitive)
a = analogRead(FLEX_PIN1); //Pinky Finger --- brown/purple 
b = analogRead(FLEX_PIN2); //Ring Finger ---  green/blue
c = analogRead(FLEX_PIN3); //Middle Finger --- orange/purple
d = analogRead(FLEX_PIN4); //Index Finger --- yellow/blue 
e = analogRead(FLEX_PIN5); //Thumb Finger --- white/purple 

  // Constrain the flex sensor values
/*a = constrain(a, sensorMin1, sensorMax1); //pinky
b = constrain(b, sensorMin2, sensorMax2);
c = constrain(c, sensorMin3, sensorMax3);
d = constrain(d, sensorMin4, sensorMax4);
e = constrain(e, sensorMin5, sensorMax5); //thumb*/

//delay(2000);                                                 //waits 2 seconds after reading the vals and displays it 2 seconds later

//Display sensor readings
/*Serial.print("Sensor Readings: ");
Serial.print("a=");   Serial.print(a);
Serial.print(", b="); Serial.print(b);
Serial.print(", c="); Serial.print(c);
Serial.print(", d="); Serial.print(d);
Serial.print(", e="); Serial.println(e);
*/

 // Update sensor data
  mpu.update();

// Read and print calibrated sensor values
//Serial.println("Accel X:\tAccel Y:\tAccel Z:");
//Serial.print(mpu.getAccX());Serial.print("\t");Serial.print(mpu.getAccY());Serial.print("\t");Serial.println(mpu.getAccZ());
 


/*  // Read and print calibrated sensor values
  Serial.print("Accel X: ");
  Serial.println(mpu.getAccX());
  Serial.print("\tY: ");
  Serial.println(mpu.getAccY());
  Serial.print("\tZ: ");
  Serial.println(mpu.getAccZ());

  Serial.print("\tGyro X: ");
  Serial.println(mpu.getGyroX());
  Serial.print("\tGyroY: ");
  Serial.println(mpu.getGyroY());
  Serial.print("\tGyroZ: ");
  Serial.println(mpu.getGyroZ());
*/
  // Delay between readings
 // delay(2000);                                 //waits 2 seconds before displaying the phrases

 if((a<sensorMax1 && (a<=726 || a==sensorMin1)) && (b<sensorMax2 && (b<=726 || b==sensorMin2)) && (c<sensorMax3 && (c<=680 || c==sensorMin3)) && (d<sensorMax4 || d<=sensorMin4) && (e>=770 && e<=800 || e>sensorMax5)) ////gx -0.35<   //gy -0.01//gz 0.03
  {
    Serial.println("Hello!");
    Btserial.println("Hello!");
  }
  
else if((a==sensorMin1 || a<=730) && (b<=sensorMin2 || b<=730) && (c==sensorMin3 || c<=675) && (d==sensorMin4 || d<=750) && (e==sensorMin5 || e<=755))
  {
    Serial.println("Bye-Bye");
    Btserial.println("Bye-Bye");
  }
  
  else if((a>=sensorMax1 || 798>a>=785 || a>=802) && (b>=770 || b>= sensorMax2) && (c>=698 || c>=sensorMax3) && (d == sensorMin4 || d<=745 || d>=729) && (e>=sensorMax5 || e>=758 || e<=765) && (gx>=-0.21))  // -0.21,-0.20,-0.19 etc...
 {
  Serial.println("Woman");
  Btserial.println("Woman");
 }
  else if((810>=a>=775 || a>=sensorMax1) && (b>=782 && b<=819 || b>= sensorMax2) && (c == sensorMin3 || c>= 668 || c<=680) && (d == sensorMin4 || (d<=745 /*&&*/ || d>=729)) && (e>=747 || e<=805 || e>=sensorMax5)) //&& (gx<=-0.41)) // -0.40,-0.39...-0.1 && 0.01,0.02,
 { 
  Serial.println("Deaf");
  Btserial.println("Deaf");
 }
 else if((a>=sensorMax1 ||  /*a>=802*/ a>=790) && (b>=770 || b>= sensorMax2) && (c>sensorMin3 || c>=sensorMax3) && (d>=sensorMax4 && d>sensorMin4) && (e>=sensorMax5 || e>=750 || e<=765 && e>sensorMin5) && (gx<=-0.01 && gx>=-0.15 /*||*/ && gx<=0.05))  // 
 { 
  Serial.println("Yes"); //DOES NOT WORK
  Btserial.println("Yes");
 }
 else if((a>=sensorMax1 || a>=795) && (b>=770 || b>= sensorMax2) && (c==sensorMin3 || c<=680) && (d==sensorMin4 || d<=750) && (e==sensorMin5 || e<=755) && (gx>=-0.16 && gx<=-0.01 || gx>=0.01))  // 
 { 
  Serial.println("No");  //DOES NOT WORK
  //Serial1.println("No"); 
 }
 else if((a>=sensorMax1 || a>=800) && (b>=800 || b>= sensorMax2) && (c>=698 || c>=sensorMax3) && (d>=sensorMax4 || d>=845) && (e>=sensorMax5 || e>=765) && (gx>=-0.92 && gx<=0.72 && gy>=-0.59 && gy<=-0.9))  // 
 { 
  Serial.println("Sorry");  //DOES NOT WORK BUT CHECK 1-2 TIMES
  //Serial1.println("Sorry");
 }
 
/*  if((e>=785 && e<=800 || e>sensorMax5) && a<sensorMax1 && b<sensorMax2 && c<sensorMax3 && d<sensorMax4)
  {
    Serial.println("Hello!");
  }

  else if((a==sensorMin1 || a<=730) && (b==sensorMin2 || b<=730) && (c==sensorMin3 || c<=675) && (d==sensorMin4 || d<=750) && (e==sensorMin5 || e<=755))
  {
    Serial.println("Bye-Bye");
  }
  else if((d == sensorMin4 || d<=745 || d>=729)  && a>=sensorMax1 && (b>=805 || b>= sensorMax2) && (c>=735 || c>=sensorMax3) && (e>=765 || e>=sensorMax5))
 {
  Serial.println("Woman");
 }
  else if ((d == sensorMin4 || (d<=745 /*&&*/ /*|| d>=729)) && (c == sensorMin3 || c>= 669 || c<=675) && (a>=795 || a>=sensorMax1) && (b>=800 || b>= sensorMax2) && (e>=765 || e>=sensorMax5)) 
 { Serial.println("Deaf");}
   
 delay(2000); */

delay(500);                       // DELAY(500); -- sweet spot for delay @ 9600 baud _wait 0.5 seconds before looping again (reading flex sensor values)
}  //END OF LOOP

void calibrateFlexSensor(int FLEX_PIN, int &sensorMin, int &sensorMax, String finger) 
{ //start of func
  Serial.print("Calibrating ");
  Serial.print(finger);
  Serial.println(".Please bend and straighten the sensor.");

  delay(3000); // Allow time for the user to prepare

  for (int i = 0; i < 10; i++) //number of iterations for calibration // i = no. of iteration
  {    
    int sensorValue = analogRead(FLEX_PIN);

    if (sensorValue < sensorMin) 
    { //read flexpin less than 1023
      sensorMin = sensorValue;
    }

    if (sensorValue > sensorMax) 
    { //read flexpin greater than 0
      sensorMax = sensorValue;
    }

    // Calculate and display ADC value, voltage, and resistance
    voltage = (sensorValue / 1023.0) * 5.0; // 5.0V is the Arduino reference voltage
    resistance = (5.0 - voltage) / (voltage / 10000.0);

    Serial.print("ADC: ");
    Serial.print(sensorValue);
    Serial.print("\tVoltage: ");
    Serial.print(voltage);
    Serial.print("V\tResistance: ");
    Serial.print(resistance);
    Serial.println(" ohms");

    delay(1000); // Small delay between readings
  } //end of for loop
} //end of func

void displayCalibrationValues(String finger, int sensorMin, int sensorMax)
{ //start of func
  Serial.print(finger);
  Serial.print("\t");
  Serial.print(sensorMin);
  Serial.print("\t");
  Serial.println(sensorMax);
} //end f=of func
